@@ -2,18 +2,19 @@
 ## solve function --- C Interface
 
 Rglpk_solve_LP <-
-function(obj, mat, dir, rhs, types = NULL, max = FALSE,
-         bounds = NULL, verbose = FALSE)
+function(obj, mat, dir, rhs, bounds = NULL, types = NULL, max = FALSE,
+          control = list(), ...)
 {
   ## validate direction of optimization
   if(!identical( max, TRUE ) && !identical( max, FALSE ))
     stop("'Argument 'max' must be either TRUE or FALSE.")
   direction_of_optimization <- as.integer(max)
 
-  ## validate verbosity flag
-  if(!identical(verbose, TRUE) && !identical(verbose, FALSE))
-    stop("'Argument 'verbose' must be either TRUE or FALSE.")
-  verb <- as.integer(verbose)
+  ## validate control list
+  dots <- list(...)
+  control[names(dots)] <- dots
+  control <- .check_control_parameters( control )
+  verb <- control$verbose
 
   ## match direction of constraints
   n_of_constraints <- length(dir)
@@ -23,6 +24,11 @@ function(obj, mat, dir, rhs, types = NULL, max = FALSE,
   if( any(is.na(direction_of_constraints)) )
     stop("Argument 'dir' must be either '<', '<=', '>', '>=' or '=='.")
 
+  ## we need to verify that obj is a numeric vector
+  ## FIXME: always use STMs?
+  if(slam::is.simple_triplet_matrix(obj))
+      obj <- as.matrix(obj)
+  obj <- as.numeric(obj)
   n_of_objective_vars <- length( obj )
 
   constraint_matrix <- as.simple_triplet_matrix(mat)
@@ -68,8 +74,11 @@ function(obj, mat, dir, rhs, types = NULL, max = FALSE,
   solution[integers | binaries] <-
     round( solution[integers | binaries])
   ## match status of solution
-  ## 0 -> optimal solution (5 in GLPK) else 1
-  status <- as.integer(x$lp_status != 5L)
+  status <- as.integer(x$lp_status)
+  if(control$canonicalize_status){
+      ## 0 -> optimal solution (5 in GLPK) else 1
+      status <- as.integer(status != 5L)
+  }
   list(optimum = sum(solution * obj), solution = solution, status = status)
 }
 
@@ -120,7 +129,7 @@ function(lp_objective_coefficients, lp_n_of_objective_vars,
     control$verbose <- FALSE
   Rglpk_solve_LP(x$objective, x$constraints[[1]],
                  x$constraints[[2]], x$constraints[[3]],
-                 x$types, x$maximum, x$bounds,
+                 types = x$types, max = x$maximum, bounds = x$bounds,
                  verbose = control$verbose)
 }
 
